<?php

namespace MasterStudy\Lms\Pro\RestApi\Repositories;

use MasterStudy\Lms\Utility\Traits\VideoTrait;

final class LessonMarkersRepository {
	use VideoTrait;

	protected $wpdb;
	protected $table_name;

	public function __construct() {
		global $wpdb;
		$this->wpdb       = $wpdb;
		$this->table_name = stm_lms_lesson_marker_questions_name( $wpdb );
	}

	/**
	 * Create a new marker for a lesson.
	 *
	 * @return bool|int Insert ID on success, false on failure.
	 */
	public function create( int $lesson_id, int $time, string $caption ) {
		$this->wpdb->insert(
			$this->table_name,
			array(
				'lesson_id' => $lesson_id,
				'marker'    => $time,
				'caption'   => $caption,
			),
			array(
				'%d',
				'%d',
				'%s',
			)
		);

		return $this->wpdb->insert_id;
	}

	public function update( int $marker_id, string $time, string $caption ) {
		$result = $this->wpdb->update(
			$this->table_name,
			array(
				'marker'  => $time,
				'caption' => $caption,
			),
			array(
				'id' => $marker_id, // Adjust the column name if your primary key is named differently
			),
			array(
				'%d',
				'%s',
			),
			array(
				'%d',
			)
		);

		return $result; // Returns the number of rows updated, or false on error
	}

	/**
	 * Retrieve markers for a given lesson.
	 *
	 * @return array Markers or empty array if none found.
	 */
	public function get_markers( int $lesson_id ): array {
		return $this->wpdb->get_results(
			$this->wpdb->prepare( //phpcs:ignore
				"SELECT * FROM {$this->table_name} WHERE lesson_id = %d", //phpcs:ignore
				$lesson_id //phpcs:ignore
			),
			ARRAY_A
		);
	}

	/**
	 * Retrieve lesson_meta and pro tip for a given lesson id.
	 *
	 * @param boolean $should_render_pro_tip
	 *
	 * @return array lesson_meta
	 */
	public function get_video_lesson_utils( int $lesson_id ): array {
		$settings     = get_option( 'stm_lms_settings' );
		$plyr_vimeo   = $settings['course_player_vimeo_video_player'] ?? false;
		$plyr_youtube = $settings['course_player_youtube_video_player'] ?? false;

		$video_type  = get_post_meta( $lesson_id, 'video_type', true );
		$lesson_meta = array();

		foreach ( $this->get_video_fields_mapping( $video_type ) as $key ) {
			$meta_value          = get_post_meta( $lesson_id, $key, true );
			$lesson_meta[ $key ] = ( 'lesson_video' === $key )
				? wp_get_attachment_url( (int) $meta_value )
				: $meta_value;
		}
		$should_render_pro_tip     = false;
		$lesson_meta['video_type'] = $video_type;

		if ( 'youtube' === $video_type || 'vimeo' === $video_type ) {
			$should_render_pro_tip = ( ! $plyr_vimeo || ! $plyr_youtube );
		}

		$current_user = wp_get_current_user();
		if ( in_array( 'stm_lms_instructor', (array) $current_user->roles ) && ! in_array( 'administrator', (array) $current_user->roles ) ) {
			$should_render_pro_tip = false;
		}

		return array(
			'shouldRenderTip' => $should_render_pro_tip,
			'lesson_metas'    => $lesson_meta,
		);
	}

	/**
	 * Delete a marker for a given lesson.
	 *
	 * @return int|false Number of rows affected or false on error.
	 */
	public function delete( int $lesson_id, int $marker_id ) {
		return $this->wpdb->delete(
			$this->table_name,
			array(
				'lesson_id' => $lesson_id,
				'id'        => $marker_id,
			),
			array(
				'%d',
				'%d',
			)
		);
	}
}
